package eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport;

import java.util.Scanner;

import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;

import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionAddSynonymToTaxon;

/**
 * This listener catches all line breaks other than carriage
 *  returns at end-of-line, and creates new name viewers,
 *  for instance, if user pastes a block of taxon information.  
 * 
 * @author p.ciardelli
 *
 */
public class NameViewerModifyListener implements ModifyListener {

	private Taxon taxon;
	
	public NameViewerModifyListener(Taxon taxon) {
		this.taxon = taxon;
	}

	public void modifyText(ModifyEvent e) {

		StyledText sText = (StyledText) e.widget;
		NameComposite nameComposite = (NameComposite) sText.getParent();
				
		// User has copy-pasted multi-line text - 
		//  split it into new source viewers
		if (sText.getText().contains("\r") || sText.getText().contains("\n")) {
			
			Scanner scanner = new Scanner( sText.getText() );
			scanner.useDelimiter (System.getProperty("line.separator"));

			nameComposite.getName().setTitleCache(scanner.next());
			while (scanner.hasNext())
				new ActionAddSynonymToTaxon(taxon, scanner.next()).run();				
			
			// Put cursor at EOL of original sourceviewer
			sText.setFocus();
			sText.setCaretOffset(sText.getText().length());
			return;
		}
				
		// Text now contains no line breaks - parse it and send it to the name composite
		nameComposite.getName().setTitleCache(sText.getText());
//		nameComposite.setName(BotanicalName.PARSED_NAME(sText.getText()));
		
//		ITaxonNameParser<BotanicalName> parser = TaxonNameParserBotanicalNameImpl.NEW_INSTANCE();	
	}
}
