// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.service.ILocationService;
import eu.etaxonomy.cdm.api.service.ILocationService.NamedAreaVocabularyType;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.description.AbsenceTerm;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.description.PresenceTerm;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.location.NamedAreaLevel;
import eu.etaxonomy.cdm.model.location.NamedAreaType;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.datasource.CdmDataSourceRepository;

/**
 * @author p.ciardelli
 * @created 02.04.2009
 * @version 1.0
 */
public class DescriptionUtil {
	private static final Logger logger = Logger
			.getLogger(DescriptionUtil.class);

	/**
	 * Returns whatever the element's title cache equivalent is,
	 * depending on its class.
	 * 
	 * @param element
	 * @param language
	 * @return 
	 */
	public static String getCache(DescriptionElementBase element, 
			Language language) {
		String cache = null;
		if (element instanceof TextData) {
			cache = ((TextData) element).getText(language);
		}
		if (element instanceof CommonTaxonName) {
			cache = ((CommonTaxonName) element).getName();
		}
		return cache == null ? "" : cache;
	}

	/**
	 * Returns whatever the element's title cache equivalent is,
	 * depending on its class, using the default language.
	 * 
	 * @param element
	 * @return 
	 */
	public static String getCache(DescriptionElementBase element) {
		return getCache(element, Language.DEFAULT());
	}
	
	/**
	 * Set whatever the element's title cache equivalent is,
	 * depending on its class.
	 * 
	 * @param element
	 * @param value
	 * @param language
	 */
	public static void setCache(DescriptionElementBase element,
			String value, Language language) {
		if (element instanceof TextData) {
			((TextData) element).putText(value, language);
			return;
		}
		if (element instanceof CommonTaxonName) {
			((CommonTaxonName) element).setName(value);
			return;
		}
		logger.warn("No matching subclass found for DescriptionElementBase object, 'cache' not set.");
	}
	
	/**
	 * Set whatever the element's title cache equivalent is,
	 * depending on its class, using the default language.
	 * 
	 * @param element
	 * @param value
	 */	
	public static void setCache(DescriptionElementBase element,
			String value) {
		setCache(element, value, Language.DEFAULT());
	}
	
	public static IDistributionInfoProvider getDistributionInfoProvider() {
		IDistributionInfoProvider provider = new DistributionInfoProvider();
		return provider;
	}
	
	static class DistributionInfoProvider implements IDistributionInfoProvider {

		private static final String TDWG = "TDWG Areas";
		private static final String ISO_COUNTRY = "ISO Country Codes";
		private static final String ISO_WATERBODY = "ISO Waterbody Codes";
		private static ILocationService locationService;
		private List<Object> vocabTypes;
		
		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IDistributionInfoProvider#getAbsenceTerms()
		 */
		@Override
		public List<AbsenceTerm> getAbsenceTerms() {
			List<AbsenceTerm> absenceTerms = new ArrayList<AbsenceTerm>();			
			absenceTerms.add(AbsenceTerm.ABSENT());
			return absenceTerms;
		}

		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IDistributionInfoProvider#getPresenceTerms()
		 */
		@Override
		public List<PresenceTerm> getPresenceTerms() {
			List<PresenceTerm> presenceTerms = new ArrayList<PresenceTerm>();
			presenceTerms.add(PresenceTerm.CULTIVATED());
			presenceTerms.add(PresenceTerm.ENDEMIC_FOR_THE_RELEVANT_AREA());
			presenceTerms.add(PresenceTerm.INTRODUCED());
			return presenceTerms;
		}

		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IDistributionInfoProvider#getVocabularyNextLevelAreas(eu.etaxonomy.cdm.model.location.NamedArea)
		 */
		@Override
		public List<NamedArea> getVocabularyNextLevelAreas(NamedArea namedArea) {
			List<NamedArea> namedAreas = new ArrayList<NamedArea>();
			namedAreas.add(NamedArea.NewInstance("USA", "USA", "USA"));
			namedAreas.add(NamedArea.NewInstance("Canada", "Canada", "CAN"));
			namedAreas.add(NamedArea.NewInstance("Mexico", "Mexico", "MEX"));
			return namedAreas;
		}

		private ILocationService getLocationService() {
			if (locationService == null) {
				locationService = TaxEditorPlugin.getDefault().getApplicationController().getLocationService();
			}
			return locationService;
		}
		
		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IDistributionInfoProvider#getVocabularyTopLevelAreas(java.lang.Object)
		 */
		@Override
		public List<NamedArea> getVocabularyTopLevelAreas(Object vocabularyType) {

			if (!(vocabularyType instanceof NamedAreaVocabularyType)) {
				return null;
			}
			for (Object o :getLocationService().getNamedAreaVocabulary(
					(NamedAreaVocabularyType) vocabularyType, NamedAreaLevel.TDWG_LEVEL1(), 
					NamedAreaType.NATURAL_AREA())) {
				logger.warn(o.toString());
			}
			
			if (vocabularyType.equals(TDWG)) {
//				for (Object o : locationService.getNamedAreaTypeVocabulary()) {
//				for (Object o : locationService.getNamedAreaLevelVocabulary()) {
//				NamedAreaVocabularyType
//				for (Object o : locationService.getNamedAreaVocabulary(vocabularyType, namedAreaLevel, namedAreaType) .getNamedAreaLevelVocabulary()) {
//					logger.warn(o.toString());
//				}
			}
			if (vocabularyType.equals(ISO_COUNTRY)) {
				
			}
			if (vocabularyType.equals(ISO_WATERBODY)) {
				
			}
			return null;
//			List<NamedArea> namedAreas = new ArrayList<NamedArea>();
//			namedAreas.add(NamedArea.NewInstance("North America", "North America", "NA"));
//			namedAreas.add(NamedArea.NewInstance("Europe", "Europe", "EUR"));
//			return namedAreas;
		}

		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IDistributionInfoProvider#getVocabularyTypes()
		 */
		@Override
		public List<Object> getVocabularyTypes() {
			if (vocabTypes == null) {
				vocabTypes = new ArrayList<Object>();
				for (NamedAreaVocabularyType vocabType: getLocationService().getNamedAreaVocabularyTypes()) {
					vocabTypes.add(vocabType);
				}
			}
				
//			vocabTypes.add("TDWG Areas");
//			vocabTypes.add("ISO Country Codes");
//			vocabTypes.add("ISO Waterbody Codes");
//			return vocabTypes;
			return vocabTypes;
		}

		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IDistributionInfoProvider#getVocabularyTypeLabel()
		 */
		@Override
		public String getVocabularyTypeLabel(Object vocabularyType) {
			if (vocabularyType instanceof String) {
				return (String) vocabularyType;
			} else {
				return vocabularyType.toString();	
			}
		}

		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IDistributionInfoProvider#getPresenceAbsenceTermLabel(eu.etaxonomy.cdm.model.description.PresenceTerm)
		 */
		@Override
		public String getPresenceAbsenceTermLabel(PresenceAbsenceTermBase term) {
			return term.getDescription();
		}

		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IDistributionInfoProvider#getNamedAreaChildren(eu.etaxonomy.cdm.model.location.NamedArea)
		 */
		@Override
		public List<NamedArea> getNamedAreaChildren(NamedArea namedArea) {
			// TODO Auto-generated method stub
			return null;
		}

		/* (non-Javadoc)
		 * @see eu.etaxonomy.taxeditor.model.IDistributionInfoProvider#getNamedAreaParent(eu.etaxonomy.cdm.model.location.NamedArea)
		 */
		@Override
		public NamedArea getNamedAreaParent(NamedArea namedArea) {
			// TODO Auto-generated method stub
			return null;
		}
	}
}
