/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.navigation;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import com.swtdesigner.SWTResourceManager;

import eu.etaxonomy.taxeditor.controller.SearchController;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * The left navigation pane.
 * 
 * @author p.ciardelli
 * @created 27.05.2008
 * @version 1.0
 */
public class SearchView extends ViewPart {
	private static final Logger logger = Logger.getLogger(SearchView.class);
	
	private Text searchText;
	private Composite searchComposite = null;
	public static final String ID = "eu.etaxonomy.taxeditor.navigation.searchview"; //$NON-NLS-1$

	private Label noResultsLabel;

	private Button searchButton;
	
	/**
	 * Create contents of the view part
	 * @param parent
	 */
	@Override
	public void createPartControl(Composite parent) {
		
		parent.setLayout(new FillLayout());

		searchComposite = new Composite(parent, SWT.NONE);
		searchComposite.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		searchComposite.setLayout(gridLayout);
		
		searchText = new Text(searchComposite, SWT.BORDER);
		searchText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		searchText.setForeground(SWTResourceManager.getColor(192, 192, 192));
		searchText.setText("Use \"*\" for wildcard searching");
		final GridData gd_useForText = new GridData(SWT.FILL, SWT.CENTER, true, false);
		searchText.setLayoutData(gd_useForText);
		searchText.addFocusListener(new FocusListener() {

			public void focusGained(FocusEvent e) {
				searchText.setForeground(SWTResourceManager.getColor(0,0,0));
				searchText.setText("");
			}

			public void focusLost(FocusEvent e) {
				if (searchText.getText() == "") {
					searchText.setForeground(SWTResourceManager.getColor(192, 192, 192));
					searchText.setText("Use \"*\" for wildcard searching");					
				}	
			}
		});
		searchText.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent e) {
				int key = e.keyCode;
				if (key == SWT.CR) {
					populateSearchResults();
				} else {
					setSearchButtonEnabled();
				}
			}
		});
		
		searchButton = new Button(searchComposite, SWT.NONE);
		searchButton.setText("Search");
		searchButton.setEnabled(false);
		searchButton.addMouseListener(new MouseAdapter() {
			
			// Populate search results resultsTable after clicking button
			public void mouseUp(MouseEvent e) {
				populateSearchResults();
			}
		});

		noResultsLabel = new Label(searchComposite, SWT.NONE);
		noResultsLabel.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
		noResultsLabel.setText("0 results found.");
		noResultsLabel.setVisible(false);
		noResultsLabel.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));

		// TEMPORARY PENDING IMPLEMENTATION - Disable visible fields
//		searchButton.setEnabled(false);
		
		createActions();
		initializeToolBar();
		initializeMenu();
	}
	
	protected void setSearchButtonEnabled() {
		if (searchText.getText().length() == 0) {
			searchButton.setEnabled(false);
		} else {
			searchButton.setEnabled(true);			
		}
	}

	protected void populateSearchResults() {
//		UiUtil.openSearchResultsView(null);

		noResultsLabel.setVisible(false);
		if (searchText.getText().length() > 0) {
			
			// Query CDM with search text
			SearchResult searchResult = CdmUtil.searchTaxaByName(searchText.getText());
			searchResult.setSearchString(searchText.getText());
//			List<TaxonNameBase> results = CdmUtil.searchNameString(searchText.getText());
			
			// If there are results, open a view in the center pane
			if (searchResult.size() > 0) {
				SearchController.openSearchResultsView(searchResult);
			} else {
			
				// Display "no results found" message
				logger.warn("no results found");
//				noResultsLabel.setText("No results found for '" + searchText.getText() + "'");
				noResultsLabel.setVisible(true);
			}				
		}
	}

	private void createActions() {
		// Create the actions
	}

	/**
	 * Initialize the toolbar
	 */
	private void initializeToolBar() {
		IToolBarManager toolbarManager = getViewSite().getActionBars()
				.getToolBarManager();
	}

	/**
	 * Initialize the menu
	 */
	private void initializeMenu() {
		IMenuManager menuManager = getViewSite().getActionBars()
				.getMenuManager();
	}

	@Override
	public void setFocus() {
		// Set the focus
	}	
}
