/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.operations.AbstractEditorOperation;

/**
 * @author p.ciardelli
 * @created 29.01.2009
 * @version 1.0
 */
public class RemoveConceptRelationOperation extends AbstractEditorOperation {
	private static final Logger logger = Logger
			.getLogger(RemoveConceptRelationOperation.class);
	
	private Taxon relatedTaxon;
	private TaxonRelationshipType relationshipType;
	private ReferenceBase citation;
	private String microcitation;
	private TaxonRelationship relationship;	

	public RemoveConceptRelationOperation(String label,
			IUndoContext undoContext, Taxon taxon, Taxon relatedTaxon,
			TaxonRelationshipType relationshipType) {
		super(label, undoContext, taxon);
		
		this.relatedTaxon = relatedTaxon;
		this.relationshipType = relationshipType;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Find relation, save citation information
		for (TaxonRelationship relationship : taxon.getTaxonRelations()) {
			if (relationship.getType().equals(relationshipType)
					&& relationship.getFromTaxon().equals(relatedTaxon)) {
				citation = relationship.getCitation();
				microcitation = relationship.getCitationMicroReference();
				this.relationship = relationship; 
			}
		}
		
		// Remove relation from taxon
		taxon.removeTaxonRelation(relationship);
				
		return redrawOpenEditor();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
//		relatedTaxon.addTaxonRelation(relationship); // don't do nothin'
		taxon.addTaxonRelation(relatedTaxon, relationshipType, citation, microcitation);
		
		return redrawOpenEditor();
	}
}
