package eu.etaxonomy.taxeditor;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;
import org.springframework.security.BadCredentialsException;
import org.springframework.security.userdetails.UsernameNotFoundException;

import eu.etaxonomy.taxeditor.dialogs.LoginDialog;
import eu.etaxonomy.taxeditor.preference.InitNomenclaturalCodePrefDialog;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

public class ApplicationWorkbenchWindowAdvisor extends WorkbenchWindowAdvisor {
	private static final Logger logger = Logger.getLogger(ApplicationWorkbenchWindowAdvisor.class);

	public ApplicationWorkbenchWindowAdvisor(IWorkbenchWindowConfigurer configurer) {
		super(configurer);
	}

	public ActionBarAdvisor createActionBarAdvisor(
			IActionBarConfigurer configurer) {
		return new ApplicationActionBarAdvisor(configurer);
	}

	public void preWindowOpen() {
		IWorkbenchWindowConfigurer configurer = getWindowConfigurer();
		configurer.setInitialSize(new Point(963, 637));
		configurer.setShowCoolBar(true);
		configurer.setShowStatusLine(true);
		configurer.setShowPerspectiveBar(true);
		configurer.setTitle("EDIT Taxonomic Editor");
		
	}
	
	public void postWindowOpen() {
		try{
			authenticate();
		}catch(BadCredentialsException e){
			logger.error("Bad credentials", e);
		}
		
		checkNomenclaturalCode();
	}

	private void checkNomenclaturalCode() {
		
		// First time Editor is opened, no nomenclatural code has been set
		if (PreferencesUtil.getPreferredNomenclaturalCode() == null) {

			logger.debug("No nomencatural code set.");
			
			Shell shell = TaxonomicEditorPlugin.getDefault().getWorkbench()
					.getActiveWorkbenchWindow().getShell();
			
			// Query user re: preferred nom. code
			Dialog dialog = new InitNomenclaturalCodePrefDialog(shell);
			dialog.open();
			
			// Short message confirming user's choice
			String code = PreferencesUtil.getPreferredNomenclaturalCodeAsString();
			MessageDialog.openInformation(shell, "Nomenclatural code set", 
					"The following has been set as your preferred nomenclatural code:\n\n\t" + 
					code + "\n\nYou can change the nomenclatural code at any time in the \"Preferences\" menu.");
		}		
	}
	
	private int authenticate(){
		Shell shell = TaxonomicEditorPlugin.getDefault().getWorkbench()
							.getActiveWorkbenchWindow().getShell();
		
		LoginDialog loginDialog = new LoginDialog(shell);
		return loginDialog.open();
	}
}
 