// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.editors.text.TextEditor;

/**
 * A list-based editor, where each line in the editor's document is associated with a domain object.
 * <p>
 * Extending classes must set:
 * <ul>
 * <li>an <code>IEntityCreator</code> to create a new domain object when the user creates a new line;
 * <li>an <code>IEntityPersistenceService</code> for interacting with the persistence layer; and
 * <li>an <code>ILineDisplayStrategy</code> for various visual manifestations of the domain object. 
 * </ul>
 * 
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class AnnotatedLineEditor extends TextEditor {
	@SuppressWarnings("unused")
	private static final Logger logger = Logger
			.getLogger(AnnotatedLineEditor.class);
	
	private IEntityCreator<?> entityCreator;
	private IEntityPersistenceService persistenceService;
	private ILineDisplayStrategy lineDisplayStrategy;
	
	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		
		AnnotatedLineEditorDocumentProvider provider = new AnnotatedLineEditorDocumentProvider();
		provider.setEntityCreator(entityCreator, input);
		provider.setPersistenceService(persistenceService, input);
		provider.setLineDisplayStrategy(lineDisplayStrategy, input);
		setDocumentProvider(provider);
		
		super.init(site, input);
	}
	
	/**
	 * @param referenceCreator
	 */
	protected void setEntityCreator(IEntityCreator<?> entityCreator) {
		this.entityCreator = entityCreator;
	}
	
	/**
	 * @param bulkEditorPersistenceService
	 */
	protected void setPersistenceService(
			IEntityPersistenceService persistenceService) {
		this.persistenceService = persistenceService;	
	}

	/**
	 * @param bulkEditorLineDisplay
	 */
	protected void setLineDisplayStrategy(
			ILineDisplayStrategy lineDisplayStrategy) {
		this.lineDisplayStrategy = lineDisplayStrategy;		
	}
	
	@Override
	protected ISourceViewer createSourceViewer(Composite parent,
			IVerticalRuler ruler, int styles) {
		
		fAnnotationAccess= getAnnotationAccess();
		fOverviewRuler= createOverviewRuler(getSharedColors());	
		LineSelectionViewer viewer = new LineSelectionViewer(parent, ruler, getOverviewRuler(), 
						isOverviewRulerVisible(), styles | SWT.WRAP);
		getSourceViewerDecorationSupport(viewer);
		
//		viewer.getTextWidget().setI
		
		return viewer;
	}
}