// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;

/**
 * @author p.ciardelli
 * @created 07.07.2009
 * @version 1.0
 */
public class BulkEditor extends AnnotatedLineEditor {
	private static final Logger logger = Logger.getLogger(BulkEditor.class);

	public static final String ID = "bulkeditor.editor";
	
	/**
	 * 
	 */
	public BulkEditor() {
		super();
		
		/**
		 * see AbstractTextEditor javadoc for explanation of context menu ids
		 */
		setEditorContextMenuId("#BulkEditorContext");

		setEntityCreator(new ReferenceCreator());
		
		setPersistenceService(new BulkEditorPersistenceService());
		
		setLineDisplayStrategy(new BulkEditorLineDisplay(this));
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.ListEditor#createSourceViewer(org.eclipse.swt.widgets.Composite, org.eclipse.jface.text.source.IVerticalRuler, int)
	 */
	protected ISourceViewer createSourceViewer(Composite parent,
			IVerticalRuler ruler, int styles) {
		ISourceViewer viewer = super.createSourceViewer(parent, ruler, styles);
		setSourceViewerConfiguration(new AnnotatedLineEditorSourceViewerConfiguration());
		addToggleMergeCandidateListener(ruler.getControl());
		
		return viewer;
	}

	/**
	 * @param control
	 */
	private void addToggleMergeCandidateListener(Control control) {
		control.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseDoubleClick(MouseEvent e) {
				StyledText textWidget = getSourceViewer().getTextWidget();
				int line = textWidget.getLineIndex(e.y);
				toggleMergeCandidateAnnotation(line);
			}
		});
	}

	/**
	 * @param line
	 */
	private void toggleMergeCandidateAnnotation(int line) {
		IDocument document = getSourceViewer().getDocument();
		LineAnnotationModel model = 
				(LineAnnotationModel) getSourceViewer().getAnnotationModel();
		
		Annotation annotation = model.getAnnotationAtLine(line, document);
		
		if (annotation != null) {
			if (annotation.getType().equals(IBulkEditorConstants.TYPE_MERGE_CANDIDATE)) {
				model.changeAnnotationType(
						annotation, LineAnnotation.TYPE_GENERIC);
			} else {
				model.changeAnnotationType(
						annotation, IBulkEditorConstants.TYPE_MERGE_CANDIDATE);
			}
		}
	}
}
