// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.text.source.IOverviewRuler;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.widgets.Composite;

/**
 * Notifies selection service every time the user changes focus to a new line.
 * 
 * @author p.ciardelli
 * @created 03.07.2009
 * @version 1.0
 */
public class LineSelectionViewer extends SourceViewer {
	@SuppressWarnings("unused")
	private static final Logger logger = Logger
			.getLogger(LineSelectionViewer.class);

	private int lastLine = 0;
	private ILineSelectionSource lineSelectionSource;
	
	/**
	 * @param parent
	 * @param verticalRuler
	 * @param overviewRuler
	 * @param showAnnotationsOverview
	 * @param styles
	 */
	public LineSelectionViewer(Composite parent, IVerticalRuler verticalRuler,
			IOverviewRuler overviewRuler, boolean showAnnotationsOverview,
			int styles) {
		super(parent, verticalRuler, overviewRuler, showAnnotationsOverview, styles);

		createLineChangeListeners();
		setLineSelectionSource(new LineSelectionSource(this));
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.TextViewer#getSelection()
	 */
	@Override
	public ISelection getSelection() {
		ISelection selection = super.getSelection();
		if (getDocument() != null) {
			return createLineSelection((TextSelection) selection);
		}
		return selection;
	}
	
	/**
	 * @param lineSelectionSource
	 */
	public void setLineSelectionSource(ILineSelectionSource lineSelectionSource) {
		this.lineSelectionSource = lineSelectionSource;
	}
	
	/**
	 * 
	 */
	private void createLineChangeListeners() {
		getTextWidget().addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				checkLineChange();
			}
		});
		getTextWidget().addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				checkLineChange();	
			}
		});
	}

	/**
	 * If the user has put the cursor on a new line, change the selection
	 */
	private void checkLineChange() {
		ISelection selection = getSelection();
		if (selection instanceof TextSelection) {
			int line = ((TextSelection) selection).getStartLine();
			if (line != lastLine) {				
				fireLineSelection();
				lastLine = line;
			}
		}
	}

	/**
	 * 
	 */
	private void fireLineSelection() {
		if (lineSelectionSource == null) {
			return;
		}
		ISelection selection = getSelection();
		if (selection instanceof TextSelection) {
			selection = createLineSelection((TextSelection) selection);
			SelectionChangedEvent event= new SelectionChangedEvent(this, selection);
			fireSelectionChanged(event);
		}
	}
	
	private LineSelection createLineSelection(TextSelection textSelection) {
		if (textSelection instanceof LineSelection) {
			return (LineSelection) textSelection;
		}
		int line = textSelection.getStartLine();
		Object selectionObject = lineSelectionSource.getSelectedObjectAtLine(line);
		return new LineSelection(textSelection, getDocument(), selectionObject);
	}
}