// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor.handler;

import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.texteditor.IDocumentProvider;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.taxeditor.bulkeditor.AnnotatedLineEditor;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorConstants;
import eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer;
import eu.etaxonomy.taxeditor.bulkeditor.LineAnnotationModel;

/**
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class MergeGroupHandler extends AbstractHandler {
	private static final Logger logger = Logger
			.getLogger(MergeGroupHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.AbstractHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
				
		IEditorPart editor = HandlerUtil.getActiveEditor(event);
		IEditorInput input = editor.getEditorInput();
		if (editor instanceof AnnotatedLineEditor && input instanceof BulkEditorInput) {

			IDocumentProvider provider = ((AnnotatedLineEditor) editor).getDocumentProvider();
			LineAnnotationModel model = 
					(LineAnnotationModel) provider.getAnnotationModel(input);
			IDocument document = provider.getDocument(input);
			
			// Check whether there are any group annotations
			Set<Annotation> candidateAnnotations = model.getAllAnnotationsOfType(IBulkEditorConstants.TYPE_MERGE_CANDIDATE);
			if (candidateAnnotations.size() == 0) {
				MessageDialog.openWarning(HandlerUtil.getActiveShell(event), 
						"No merge candidates", "No objects have been chosen for merging.");
				return null;				
			}
			
			// Check whether group merge target has been set
			Annotation targetAnnotation = model.getFirstAnnotationOfType(IBulkEditorConstants.TYPE_MERGE_TARGET);
			if (targetAnnotation == null) {
				MessageDialog.openWarning(HandlerUtil.getActiveShell(event), 
						"No group merge target set", "No group merge target has been set.");
				return null;
			}			
			Object targetEntity = ((IEntityContainer<?>) targetAnnotation).getEntity();
			
			logger.info("Merging group");
			model.printAnnotations();
			for (Annotation annotation : candidateAnnotations) {
				try {
					// Remove text from document
					Position position = model.getPosition(annotation);
					document.replace(position.getOffset(), position.getLength(), "");
					
					// Mark entity container for merging with target entity
					((IEntityContainer) annotation).markAsMerged(targetEntity);
					logger.info("Merging " + annotation + " with " + targetAnnotation);
					
					// Remove annotation from model
					model.removeAnnotation(annotation);
				} catch (BadLocationException e) {
					// Do nothing
				}
			}
			model.printAnnotations();	
			
			model.removeTypeFromAllAnnotations(IBulkEditorConstants.TYPE_MERGE_CANDIDATE);
			model.removeTypeFromAllAnnotations(IBulkEditorConstants.TYPE_MERGE_TARGET);
		}
		return null;
	}
}