// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.referencingobjects;

import java.lang.reflect.InvocationTargetException;
import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.IProgressService;

import eu.etaxonomy.cdm.api.service.ICommonService;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.LanguageStringBase;
import eu.etaxonomy.cdm.model.common.OriginalSource;
import eu.etaxonomy.cdm.model.common.ReferencedEntityBase;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.taxeditor.propertysheet.reference.ReferencePropertySource;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 * @created 09.07.2009
 * @version 1.0
 */
public class ReferencingObjectsService implements
		IReferencingObjectsService {
	private static final Logger logger = Logger
			.getLogger(ReferencingObjectsService.class);

	private Set<CdmBase> referencingObjects;
	static final Set<CdmBase> EMPTY_SET = new HashSet<CdmBase>();
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IReferencedObjectsService#getReferenceObject(java.lang.Object)
	 */
	public CdmBase getReferencedObject(Object entity) {
		if (entity  instanceof ReferencePropertySource) {
			return ((ReferencePropertySource) entity).getReference();
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IReferencedObjectsService#getReferencedObjects(java.lang.Object)
	 */
	public Set<CdmBase> getReferencingObjects(Object entity) {
		final CdmBase referencedObject = getReferencedObject(entity);
		referencingObjects = EMPTY_SET;
		if (referencedObject != null) {
			IWorkbench wb = PlatformUI.getWorkbench();
			IProgressService ps = wb.getProgressService();
			try {
				ps.busyCursorWhile(new IRunnableWithProgress() {
					public void run(IProgressMonitor pm) {
						ICommonService commonService = CdmStore.getApplicationController().getCommonService();
						referencingObjects = commonService.getReferencingObjects(referencedObject);
					}
				});
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
//			IWorkbench wb = PlatformUI.getWorkbench();
//			   try {
//				   wb.getActiveWorkbenchWindow().run(true, true, new IRunnableWithProgress() {
//					public void run(IProgressMonitor monitor)
//							throws InvocationTargetException,
//							InterruptedException {
//						ICommonService commonService = CdmStore.getApplicationController().getCommonService();
//						monitor.beginTask("Retrieving referencing objects:", 1);
//						referencingObjects = commonService.getReferencingObjects(referencedObject);
//						monitor.done();
//					}
//				});
//			} catch (InvocationTargetException e) {
//				// Nichts
//			} catch (InterruptedException e) {
//				// User canceled the operation... just ignore.
//			}

			
			
		}
		return referencingObjects;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.referencingobjects.IReferencingObjectsService#getObjectClass(java.lang.Object)
	 */
	public String getObjectClass(Object element) {
		return element.getClass().getSimpleName();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.referencingobjects.IReferencingObjectsService#getObjectDescription(java.lang.Object)
	 */
	public String getObjectDescription(Object element) {
		if (element instanceof IdentifiableEntity) {
			return ((IdentifiableEntity) element).getTitleCache();
		}
		if (element instanceof ReferencedEntityBase) {
			return ((ReferencedEntityBase) element).getOriginalNameString();
		}
		if (element instanceof LanguageStringBase) {
			return ((LanguageStringBase) element).getText();
		}
		return null;
	}
}
