// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.referencingobjects;

import java.util.ArrayList;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.part.ViewPart;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.taxeditor.bulkeditor.LineSelection;

/**
 * @author p.ciardelli
 * @created 08.07.2009
 * @version 1.0
 */
public class ReferencingObjectsView extends ViewPart {
	private static final Logger logger = Logger
			.getLogger(ReferencingObjectsView.class);

	public static final String ID = "taxeditor-bulkeditor.referencingobjectsview";

	private IReferencingObjectsService referencingObjectsService;
	private Set<CdmBase> referencingObjects;
	private CdmBase referencedObject;
	
	private TableViewer referencingObjectsViewer;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {
		referencingObjectsService = new ReferencingObjectsService();
		createReferencingObjectsViewer(parent);
	}

    /**
     * @param parent
     */
    private void createReferencingObjectsViewer(Composite parent) {
        referencingObjectsViewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
									| SWT.V_SCROLL | SWT.FULL_SELECTION);
        createColumns(referencingObjectsViewer);
        referencingObjectsViewer.setContentProvider(new ReferencingObjectsContentProvider());
        referencingObjectsViewer.setLabelProvider(new ReferencingObjectsLabelProvider(referencingObjectsService));
        referencingObjectsViewer.setInput(new ArrayList());
    }
	
	// This will create the columns for the table
	private void createColumns(TableViewer viewer) {
		Table table = viewer.getTable();
		String[] titles = {"Class", "Description" };
		int[] bounds = { 100, 200 };

		for (int i = 0; i < titles.length; i++) {
			TableViewerColumn column = new TableViewerColumn(viewer, SWT.NONE);
			column.getColumn().setText(titles[i]);
			column.getColumn().setWidth(bounds[i]);
			column.getColumn().setResizable(true);
			column.getColumn().setMoveable(true);
		}
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
	}
    
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
		referencingObjectsViewer.getControl().setFocus();
	}

	/**
	 * 
	 */
	public void updateReferencingObjects() {
        try {
            IEditorPart editor = getSite().getPage().getActiveEditor();
            ISelection selection = editor.getEditorSite().getSelectionProvider()
                                         .getSelection();

            if ((selection != null) && selection instanceof LineSelection) {
            	Object entity = ((IStructuredSelection) selection).getFirstElement();
            	CdmBase referencedObject = referencingObjectsService.getReferencedObject(entity);
            	if (referencedObject != null) {
            		this.referencedObject = referencedObject;
            		this.referencingObjects = referencingObjectsService.getReferencingObjects(entity);
            		updateView();
            	}	
            }
        } catch (Exception e) {
            logger.error(e.getStackTrace());
        }
	}

	/**
	 * 
	 */
	private void updateView() {
		referencingObjectsViewer.setInput(referencingObjects);
		setContentDescription("'" + referencedObject + "' is referenced by:");
	}
}
