/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import org.apache.log4j.Logger;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;

import eu.etaxonomy.taxeditor.editor.description.DescriptionElementComposite;

/**
 * Draws a border on a Composite on an IManagedForm when it gets focus, 
 *  undraws it when the Composite loses focus.
 *  
 * @author p.ciardelli
 *
 */
public class CompositeBorderDecorator implements FocusListener {	
	private static final Logger logger = Logger.getLogger(CompositeBorderDecorator.class);
		
	private FormToolkit toolkit;
	private ScrolledForm scrolledForm;
	private Composite borderedComposite;
	
	/**
	 * If true, the <code>borderedComposite</code>'s border is erased when
	 * it loses focus.
	 */
	private boolean doLoseFocus = true;

	public CompositeBorderDecorator(Composite composite, IManagedForm form) {
				
		this.borderedComposite = composite;
		this.toolkit = form.getToolkit();
		this.scrolledForm = form.getForm();
		
		// Make sure composite retains its color scheme after being adapted to toolkit
		Color backgroundColor = composite.getBackground();
		
		toolkit.adapt(composite);
		
		composite.setBackground(backgroundColor);
		
	}
	
	
	public void focusGained(FocusEvent e) {
		paintBorder();
	}

	
	public void focusLost(FocusEvent e) {
		if (doLoseFocus) {
			unpaintBorder();
		}
	}
	
	public void setBorderedComposite(Composite borderedComposite) {
		this.borderedComposite = borderedComposite;
	}
	
	/**
	 * If this is set to true, border will disappear when focus is lost.
	 * Otherwise, it will remain until explicitly erased. 
	 * 
	 * @param doLoseFocus
	 */
	public void setLoseFocus(boolean doLoseFocus) {
		this.doLoseFocus = doLoseFocus;
	}
	
	/**
	 * Removes the border from <code>borderedComposite</code>.
	 */
	public void unpaintBorder() {
		logger.warn("unpainting " + borderedComposite);
		paintBorder(null);		
	}
	
	/**
	 * Adds a border to <code>borderedComposite</code>.
	 */
	public void paintBorder() {
		logger.warn("painting " + borderedComposite.getParent());
		paintBorder(FormToolkit.TEXT_BORDER);
	}
	
	private void paintBorder(String border) {		
		borderedComposite.setData(FormToolkit.KEY_DRAW_BORDER, border);
		toolkit.paintBordersFor(borderedComposite.getParent());
		scrolledForm.reflow(false);
	}
}