/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.taxeditor.editor.description.TaxonDescriptionEditor;
import eu.etaxonomy.taxeditor.editor.images.TaxonImageEditor;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;

/** 
 * @author p.ciardelli
 * @created 08.01.2009
 * @version 1.0
 */
public interface IFreeTextElementFactory {

	/* Taxon name editor elements */
	public Composite createAcceptedTaxon(AbstractTaxonEditor editor);
	
	public Composite createSynonym(TaxonNameEditor editor, Synonym synonym);

	public Composite createMisappliedName(TaxonNameEditor editor, Taxon misappliedName);
	
	public Composite createHomotypicalGroup(AbstractTaxonEditor editor, HomotypicalGroup group);
	
	public Composite createMisappliedGroup(TaxonNameEditor editor);
	
	public Composite createConceptGroup(TaxonNameEditor editor);

	public Composite createConcept(TaxonNameEditor editor, TaxonRelationship relationship);
	
	
	/* Taxon description editor elements */
	public Composite createDescriptionLabel(AbstractTaxonEditor editor, TaxonDescription description);
	
	public Composite createDescriptionGroup(AbstractTaxonEditor editor, TaxonDescription description);
	
	public Composite createDescriptionElement(TaxonDescriptionEditor editor, TaxonDescription description, DescriptionElementBase element);
		
	public Composite createDescriptionFeatureGroup(TaxonDescriptionEditor editor, TaxonDescription description, Feature feature);

	
	/* Taxon images editor elements */
	public Composite createGalleryLabel(AbstractTaxonEditor editor, TaxonDescription imageGallery);

	public Composite createImageElement(TaxonImageEditor editor, ImageFile imageFile);


}
