/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.description;

import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.taxeditor.editor.GroupedComposite;
import eu.etaxonomy.taxeditor.model.DescriptionHelper;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.Resources;
import eu.etaxonomy.taxeditor.propertysheet.description.DescriptionElementPropertySource;

/**
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class DescriptionElementComposite extends GroupedComposite {

	/**
	 * ************ COMPOSITE TYPES ************
	 */
	public static final String TEXT_DATA = "text_data";
	public static final String COMMON_TAXON_NAME = "common_taxon_name";
	public static final String DISTRIBUTION = "distribution";

	/**
	 * ************ FONTS ************
	 */
	public static final Font ELEMENT_FONT = Resources
			.getFont(Resources.SYNONYM_FONT);

	/**
	 * ************ ICONS ************
	 */
	public static final Image BLACK_SQUARE_ICON = ImageResources
			.getImage(ImageResources.BLACK_SQUARE_ICON);

	protected DescriptionElementBase element;

	private static final String EMPTY_NAME_PROMPT = "Click to add descriptive element text";

	public DescriptionElementComposite(TaxonDescriptionEditor editor, Composite parent, DescriptionElementBase element) {
		super(editor, parent);
		
		this.element = element;
		
		
		createTextViewer();
		createBorderSupport();
		createLineWrapSupport();
		
		createParser();
		createEmptyViewerPrompt(EMPTY_NAME_PROMPT);
		
		setIsDraggable(false);
		setIcon(BLACK_SQUARE_ICON);
		setFont(getViewerFont());

		//createMenu();
		this.setMenu(editor.getMenu());
		
		initTextViewer();
	}
	
	protected void initTextViewer() {
		
		String text = null;

		text = DescriptionHelper.getCache(element);
		
		if (text.length() == 0) {
			initEmptyText();
		} else {
			textViewer.getTextWidget().setText(text);
		}
	}

	public DescriptionElementBase getElement() {
		return element;
	}
	
	protected void parse(String text) {
		
		DescriptionHelper.setCache(getElement(), text);
		
		// Manually refresh the property sheet to reflect changes
		setSelection();
		
		setDirty(true);
	}
	
	public IPropertySource getPropertySource() {
		return new DescriptionElementPropertySource(getElement());
	}

	@Override
	protected Font getViewerFont() {
		return ELEMENT_FONT;
	}
	
	public Object getData () {
		return getElement();
	}
}