/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.description;

import org.apache.log4j.Logger;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.taxeditor.editor.AbstractTaxonEditor;
import eu.etaxonomy.taxeditor.editor.GroupedComposite;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.Resources;
import eu.etaxonomy.taxeditor.propertysheet.description.TaxonDescriptionPropertySource;

/**
 * Composite for editing a description's label / Title Cache information.
 * 
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 09.09.2008
 * @version 1.0
 */
public class DescriptionLabelComposite extends GroupedComposite {
	private static final Logger logger = Logger
			.getLogger(DescriptionLabelComposite.class);

	/**
	 * ************ FONTS ************
	 */
	public static final Font DESCRIPTION_LABEL_FONT = Resources
			.getFont(Resources.ACCEPTED_TAXON_FONT);
	
	/**
	 * ************ ICONS ************
	 */
	public static final Image DESCRIPTION_LABEL_ICON = ImageResources
			.getImage(ImageResources.BLACK_SQUARE_ICON);	
	
	private static final String EMPTY_NAME_PROMPT = "Click to add description label";

	private TaxonDescription description;
	
	public DescriptionLabelComposite(AbstractTaxonEditor editor, Composite parent, TaxonDescription description) {
		super(editor, parent);
		
		this.description = description;
		
		
		createTextViewer();
		createBorderSupport();
		createLineWrapSupport();

		createParser();
		createEmptyViewerPrompt(EMPTY_NAME_PROMPT);
		
		setIsDraggable(false);
		setFont(DESCRIPTION_LABEL_FONT);
		setIcon(DESCRIPTION_LABEL_ICON);
		
		this.setMenu(editor.getMenu());
		
		initTextViewer();
	}
	
	private void initTextViewer() {
		if (textViewer != null) {
			String text = getDescription().getTitleCache();
			
			if (text.length() == 0) {
				initEmptyText();
			} else {
				textViewer.getTextWidget().setText(text);
			}
		} else {
			logger.warn("TextViewer is null.");
		}
	}

	public TaxonDescription getDescription() {
		return description;
	}

	protected void parse(String text) {
		getDescription().setTitleCache(text);
		
		// Manually refresh the property sheet to reflect changes
		setSelection();
		
		setDirty(true);		
	}
	
	public IPropertySource getPropertySource() {
		return new TaxonDescriptionPropertySource(getDescription());
	}

	@Override
	protected Font getViewerFont() {
		return DESCRIPTION_LABEL_FONT;
	}
	
	public Object getData () {
		return getDescription();
	}
}