/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.description;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.TableWrapData;

/**
 * @author p.ciardelli
 *
 */
public class DescriptionViewer extends Composite {

	private Text textWidget;
	private Label iconWidget;
	private List<FocusListener> focusListeners = new ArrayList<FocusListener>();
	private List<ModifyListener> modifyListeners = new ArrayList<ModifyListener>();
	private List<MouseListener> mouseListeners = new ArrayList<MouseListener>();

	public DescriptionViewer(Composite parent, int style, IManagedForm managedForm) {
		super(parent, style);
		
		setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		
		GridLayout layout = new GridLayout(2, false);
		layout.marginBottom = 2;
		layout.marginTop = 2;
		layout.marginLeft = 2;
		layout.marginRight = 2;
		layout.verticalSpacing = 2;
		setLayout(layout);
		
//		managedForm.getToolkit().adapt(this);
		
		iconWidget = new Label(this, style);
//		textWidget = new Text(this, style);
		textWidget = managedForm.getToolkit().createText(this, "", style);
		textWidget.setData(FormToolkit.KEY_DRAW_BORDER, FormToolkit.TEXT_BORDER);
		
		GridData iconGridData = new GridData();
		iconGridData.widthHint = 8;
		iconGridData.heightHint = 8;
		iconGridData.verticalIndent = 4;
		iconGridData.verticalAlignment = SWT.TOP;
		iconWidget.setLayoutData(iconGridData);
		
		textWidget.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
//		iconWidget.setBackground(parent.getDisplay().getSystemColor(SWT.COLOR_BLACK));
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Control#setFont(org.eclipse.swt.graphics.Font)
	 */
	@Override
	public void setFont(Font font) {
		textWidget.setFont(font);
	}
	
	/**
	 * @param text
	 */
	public void setText(String text) {
		textWidget.setText(text);
	}

	/**
	 * @param icon
	 */
	public void setIcon(Image icon) {
		iconWidget.setBackgroundImage(icon);
	}

	/**
	 * @return
	 */
	public String getText() {
		return textWidget.getText();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Control#addFocusListener(org.eclipse.swt.events.FocusListener)
	 */
	@Override
	public void addFocusListener(FocusListener listener) {
		focusListeners.add(listener);
		iconWidget.addFocusListener(listener);
		textWidget.addFocusListener(listener);
		super.addFocusListener(listener);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Conewntrol#addMouseListener(org.eclipse.swt.events.MouseListener)
	 */
	@Override
	public void addMouseListener(MouseListener listener) {
		mouseListeners.add(listener);
		iconWidget.addMouseListener(listener);
		textWidget.addMouseListener(listener);
		super.addMouseListener(listener);
	}

	/**
	 * @param parseListener
	 */
	public void addModifyListener(ModifyListener listener) {
		modifyListeners.add(listener);
		textWidget.addModifyListener(listener);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Widget#dispose()
	 */
	@Override
	public void dispose() {
		
		for (FocusListener listener : focusListeners) {
			textWidget.removeFocusListener(listener);
			iconWidget.removeFocusListener(listener);
		}
		for (MouseListener listener : mouseListeners) {
			textWidget.removeMouseListener(listener);
			iconWidget.removeMouseListener(listener);
		}
		for (ModifyListener listener : modifyListeners) {
			textWidget.removeModifyListener(listener);
		}
		super.dispose();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Control#setMenu(org.eclipse.swt.widgets.Menu)
	 */
	@Override
	public void setMenu(Menu menu) {
		textWidget.setMenu(menu);
		iconWidget.setMenu(menu);
		super.setMenu(menu);
	}

	/**
	 * @param editable
	 */
	public void setEditable(boolean editable) {
		textWidget.setEditable(editable);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Control#setBackground(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setBackground(Color color) {
		iconWidget.setBackground(color);
		textWidget.setBackground(color);
		super.setBackground(color);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Widget#setData(java.lang.String, java.lang.Object)
	 */
	@Override
	public void setData(String key, Object value) {
		if (FormToolkit.KEY_DRAW_BORDER.equals(key)) {
			textWidget.setData(key, value);
		} else {
			super.setData(key, value);
		}
//		super.setData(key, value);
	}
}