/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.images;

import org.apache.log4j.Logger;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.taxeditor.editor.AbstractTaxonEditor;
import eu.etaxonomy.taxeditor.editor.GroupedComposite;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.Resources;
import eu.etaxonomy.taxeditor.propertysheet.description.TaxonDescriptionPropertySource;

/**
 * Composite for editing a description's label / Title Cache information.
 * 
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 09.09.2008
 * @version 1.0
 */
public class GalleryLabelComposite extends GroupedComposite {
	private static final Logger logger = Logger
			.getLogger(GalleryLabelComposite.class);

	/**
	 * ************ FONTS ************
	 */
	public static final Font DESCRIPTION_LABEL_FONT = Resources
			.getFont(Resources.ACCEPTED_TAXON_FONT);
	
	/**
	 * ************ ICONS ************
	 */
	public static final Image DESCRIPTION_LABEL_ICON = ImageResources
			.getImage(ImageResources.BLACK_SQUARE_ICON);	
	
	private static final String EMPTY_NAME_PROMPT = "Click to add label for image gallery";

	private TaxonDescription description;
	
	public GalleryLabelComposite(AbstractTaxonEditor editor, Composite parent, TaxonDescription description) {
		super(editor, parent);
		
		this.description = description;
		
		
		createTextViewer();
		createBorderSupport();
		createLineWrapSupport();

//		createParser();
		createEmptyViewerPrompt(EMPTY_NAME_PROMPT);
		
		setIsDraggable(false);
		setFont(DESCRIPTION_LABEL_FONT);
		setIcon(DESCRIPTION_LABEL_ICON);
		
		this.setMenu(editor.getMenu());
		
		initTextViewer();
	}
	
	private void initTextViewer() {
		if (textViewer != null) {
			String text = getDescription().getTitleCache();
			
			if (text.length() == 0) {
				initEmptyText();
			} else {
				textViewer.getTextWidget().setText(text);
			}
		} else {
			logger.warn("TextViewer is null.");
		}
	}

	public TaxonDescription getDescription() {
		return description;
	}

//	/**
//	 * FIXME migrate menu to plugin.xml configuration
//	 */
//	private void createMenu() {
//		
//		MenuManager featuresMenu = new MenuManager("Add description element");
//		featuresMenu.setRemoveAllWhenShown(true);
//		featuresMenu.addMenuListener(new IMenuListener() {
//			public void menuAboutToShow(IMenuManager manager) {
//				for (final Feature feature : PreferencesUtil.getPreferredFeatures()) {
//					
//					String text = feature.getLabel();
//					manager.add(new Action(text){
//						
//						public void run() {
//							IUndoableOperation operation = new CreateDescriptionElementOperation
//									(this.getText(), editor.getUndoContext(), taxon, description, feature, editor);
//							EditorUtil.executeOperation(operation);
//						}
//					});
//				}
//			}
//		});
//		
//		// Important: add an empty action. Otherwise, when the main menu sees that
//		// there are no actions in the submenu, it will not be shown
//		// HACK?
//		featuresMenu.add(new Action() {});
//
//		ContextMenu contextMenu = createContextMenu();
//		contextMenu.addSubmenu(featuresMenu);
//	}

	protected void parse(String text) {
		getDescription().setTitleCache(text);
		
		// Manually refresh the property sheet to reflect changes
		setSelection();
		
		setDirty(true);		
	}
	
	public IPropertySource getPropertySource() {
		return new TaxonDescriptionPropertySource(getDescription());
	}

	@Override
	protected Font getViewerFont() {
		return DESCRIPTION_LABEL_FONT;
	}
	
	public Object getData () {
		return getDescription();
	}
}