// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.images.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.Page;
import eu.etaxonomy.taxeditor.editor.images.GalleryLabelComposite;
import eu.etaxonomy.taxeditor.editor.images.TaxonImageEditor;
import eu.etaxonomy.taxeditor.operations.DeleteImageOperation;
import eu.etaxonomy.taxeditor.propertysheet.images.ImagePropertySource;

/**
 * @author n.hoffmann
 * @created 27.04.2009
 * @version 1.0
 */
public class DeleteImageHandler extends AbstractHandler implements IHandler {
	private static final Logger logger = Logger
			.getLogger(DeleteImageHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		MultiPageTaxonEditor editor = EditorUtil.getActiveEditor();
		
		TaxonImageEditor page = (TaxonImageEditor) editor.getPage(Page.IMAGE);
		
		ISelection menuSelection = HandlerUtil.getActiveMenuSelection(event);
		
		// FIXME right now we have only one description per taxon
		// this will change in the future and then attaching the feature to
		// the firstGroupedComposite will not work anymore
		TaxonDescription description = ((GalleryLabelComposite) page.getFirstGroupedComposite()).getDescription();
		
		
		ImageFile imageFile = ((ImagePropertySource) ((StructuredSelection) menuSelection).getFirstElement()).getImageFile();
		
		IUndoableOperation operation;
		try {
			// we are not passing the description and thereby assume that all occurrences of the image
			// should be deleted
			operation = new DeleteImageOperation(event.getCommand().getName(), 
					editor.getUndoContext(), editor.getTaxon(), description, imageFile, editor);
			EditorUtil.executeOperation(operation);
		} catch (NotDefinedException e) {
			logger.warn("Command name not set");
		}
		
		return null;
	}
}
