/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.labels;

import org.apache.log4j.Logger;
import org.eclipse.swt.graphics.Image;

import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * Returns labels and images that are dependent on a nomenclatural
 * code using a code-specific ILabelImageStrategy.
 * 
 * @author p.ciardelli
 *
 */
public class LabelImageProvider implements ILabelImageStrategy {
	private static final Logger logger = Logger.getLogger(LabelImageProvider.class);

	private ILabelImageStrategy strategy;

	public static ILabelImageStrategy getLabelStrategy(TaxonNameBase name) {
		if (name == null) {
			if (PreferencesUtil.getPreferredNomenclaturalCode().
					equals(NomenclaturalCode.ICZN)) {
				return new ZoologicalNameLabelStrategy();
			}
		} else if (name.getNomenclaturalCode() != null &&
				name.getNomenclaturalCode().equals(NomenclaturalCode.ICZN)) {
			return new ZoologicalNameLabelStrategy();
		}
		return new DefaultLabelStrategy();
	}

	public LabelImageProvider(ILabelImageStrategy strategy) {
		this.strategy = strategy;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.labels.IEditorLabelStrategy#getNameRelationTypeLabel(eu.etaxonomy.cdm.model.name.NameRelationshipType)
	 */
	public String getNameRelationTypeLabel(NameRelationshipType type) {
		return strategy.getNameRelationTypeLabel(type);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.labels.IEditorLabelStrategy#getNameRelationInverseTypeLabel(eu.etaxonomy.cdm.model.name.NameRelationshipType)
	 */
	public String getNameRelationTypeInverseLabel(NameRelationshipType type) {
		return strategy.getNameRelationTypeInverseLabel(type);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.labels.ILabelImageStrategy#getNameRelationImage(eu.etaxonomy.cdm.model.name.NameRelationship)
	 */
	public Image getNameRelationImage(NameRelationship relation) {
		return strategy.getNameRelationImage(relation);
	}
}
