/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.operations.ChangeConceptToSynonymOperation;
import eu.etaxonomy.taxeditor.propertysheet.name.MisapplicationPropertySource;

/**
 * @author p.ciardelli
 * @created 13.01.2009
 * @version 1.0
 */
public class MisappliedNameComposite extends NameComposite<Taxon> {
	
	public MisappliedNameComposite(TaxonNameEditor editor, Composite parent, Taxon misappliedName) {
		super(editor, parent, misappliedName);

	}
	

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.NameComposite#initializeComposite()
	 */
	@Override
	protected void initializeComposite() {
		setIsDraggable(true);
		setIcon(MISAPPLIEDNAME_ICON);
		setFont(getViewerFont());
		setIndent(MISAPPLIEDNAME_INDENT);
		
		showSec();		
		
		initTextViewer(getMisappliedName());
	}

	private void showSec() {
		if (getMisappliedName() == null) {
			return;
		}
		
		if (getMisappliedName().getSec() == null) {
			setNonEditableInfo("sec. ???");
		} else {
			setNonEditableInfo("sec. " + getMisappliedName().getSec().getTitleCache());
		}
	}
	
	public Taxon getMisappliedName() {
		return getTaxonBase();
	}
		
	public boolean setParent(Composite parent) {

		boolean doSetParent = false;	

		// Has this been moved to a HomotypicalGroup?
		if (parent instanceof HomotypicalGroupComposite) {
			
			HomotypicalGroup homotypicalGroup = 
					((HomotypicalGroupComposite)parent).getGroup();
			
			IUndoableOperation operation = new ChangeConceptToSynonymOperation
					("change misapplication to synonym", editor.getUndoContext(), taxon, getMisappliedName(), homotypicalGroup, editor);
			EditorUtil.executeOperation(operation);
			
			doSetParent = true;
		}
		return doSetParent;
	}
	
	public IPropertySource getPropertySource() {
		return new MisapplicationPropertySource(getMisappliedName());
	}

	@Override
	protected Font getViewerFont() {
		return MISAPPLIEDNAME_FONT;
	}
	
	protected void calculateErrors() {
		super.calculateErrors();
		
		textViewer.setShowSecError(getMisappliedName());
	}

	@Override
	protected TaxonNameBase<?, ?> getName() {
		return getMisappliedName().getName();
	}
	
	public Object getData () {
		return getMisappliedName();
	}

}