/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.propertysheet.name;

import java.util.Collection;
import java.util.Set;

import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.taxeditor.editor.labels.ILabelImageStrategy;
import eu.etaxonomy.taxeditor.editor.labels.LabelImageProvider;
import eu.etaxonomy.taxeditor.propertysheet.CollectionPropertySource;

public class NameRelationsPropertySource extends CollectionPropertySource {
	
	private TaxonNameBase<?, ?> name;

	private Set<?> nameRelations;
	private LabelImageProvider labelProvider;

	public NameRelationsPropertySource(TaxonNameBase<?, ?> name, Collection<?> collection) {
		super(collection, name);
		this.name = name;
				
		this.setNameRelations(name.getNameRelations());
	}
	
	@Override
	protected String getItemDisplayName(Object item) {
		String itemDisplayName = "";
		
		if (item instanceof NameRelationship) {
			
			NameRelationship nameRelation = (NameRelationship) item;
						
			if (nameRelation.getType() != null) {
				NameRelationshipType type = nameRelation.getType();
				String label;
				if (nameRelation.getFromName().equals(getCollectionOwner())) {
					label = getLabelProvider().getNameRelationTypeLabel(type);
				} else {
					label = getLabelProvider().getNameRelationTypeInverseLabel(type);
				}
				itemDisplayName = label;
			} else {
				itemDisplayName = "Relation has no type";
			}
		}
		return itemDisplayName;
	}

	/**
	 * @return
	 */
	private ILabelImageStrategy getLabelProvider() {
		if (labelProvider == null) {
			ILabelImageStrategy strategy = LabelImageProvider.getLabelStrategy(name);
			this.labelProvider = new LabelImageProvider(strategy);
		}
		return labelProvider;
	}

	@Override
	public Object getPropertyValue(Object id) {
		if (id instanceof NameRelationship) {
			TaxonNameBase<?, ?> relatedName;
			if (((NameRelationship) id).getFromName().equals(name)) {
				relatedName = ((NameRelationship) id).getToName();
			} else {
				relatedName = ((NameRelationship) id).getFromName();
			}
		return relatedName.getTitleCache();
		}
		return null;
	}

	@Override
	public void setPropertyValue(Object id, Object value) {
		// Fields not editable in property sheet view
	}

	@Override
	public String toString() {
		// "Name Relations" header has no value
		return "";
	}

	/**
	 * @param nameRelations the nameRelations to set
	 */
	public void setNameRelations(Set<?> nameRelations) {
		this.nameRelations = nameRelations;
	}

	/**
	 * @return the nameRelations
	 */
	public Set<?> getNameRelations() {
		return nameRelations;
	}
}