// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.name;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.agent.INomenclaturalAuthor;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.taxeditor.propertysheet.ICdmBasePropertySource;

/**
 * @author p.ciardelli
 * @created 18.05.2009
 * @version 1.0
 */
public class NomenclaturalAuthorshipPropertySource implements ICdmBasePropertySource {

	// Property unique keys
	public static final String P_ID_EDITABLECACHE = "P_ID_EDITABLECACHE";		
	public static final String P_ID_PROTECT_CACHE = "P_ID_PROTECT_CACHE";
	public static final String P_ID_AUTHORSHIPCACHE = "P_ID_AUTHORSHIPCACHE";
	public static final String P_ID_EXAUTHORSHIPCACHE = "P_ID_EXAUTHORSHIPCACHE";
	public static final String P_ID_BASAUTHORSHIPCACHE = "P_ID_BASAUTHORSHIPCACHE";
	public static final String P_ID_EXBASAUTHORSHIPCACHE = "P_ID_EXBASAUTHORSHIPCACHE";
	
    // Property display keys
	public static final String P_EDITABLECACHE = "Editable Cache";
	public static final String P_PROTECT_CACHE = "Protect Cache from overwriting?";
	public static final String P_AUTHORSHIPCACHE = "Authorship";
	public static final String P_EXAUTHORSHIPCACHE = "Ex-Authorship";
	public static final String P_BASAUTHORSHIPCACHE = "Basionym Authorship";
	public static final String P_EXBASAUTHORSHIPCACHE = "Ex-Basionym Authorship";

	private static final int CACHE_NOT_PROTECTED = 0;
	private static final int CACHE_PROTECTED = 1;
	
	private NonViralName name;
	
	public NomenclaturalAuthorshipPropertySource(NonViralName name) {
		this.name = name;

		initDescriptors();
	}
	
	protected void initDescriptors() {
		
		List<String> displayFields = new ArrayList<String>();
			
		// Cache fields
		displayFields.add(P_ID_EDITABLECACHE);
		displayFields.add(P_ID_PROTECT_CACHE);
		
    	displayFields.add(P_ID_AUTHORSHIPCACHE);
    	displayFields.add(P_ID_EXAUTHORSHIPCACHE);
    	displayFields.add(P_ID_BASAUTHORSHIPCACHE);
    	displayFields.add(P_ID_EXBASAUTHORSHIPCACHE);
    
		
		for (String field : displayFields) {
			addDescriptor(field);
		}
	}

	protected Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
	
	protected void addDescriptor(String id) {
				
		// Editable cache
		if (id.equals(P_ID_EDITABLECACHE)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_EDITABLECACHE, P_EDITABLECACHE));
		}
		
		// Protect cache?
		if (id.equals(P_ID_PROTECT_CACHE)) {
			descriptors.addElement(
					new ComboBoxPropertyDescriptor(P_ID_PROTECT_CACHE, P_PROTECT_CACHE, 
							new String[] {"no", "yes"}));
		}		
		
		boolean isProtectedCache = name.isProtectedAuthorshipCache();
		
		// Authorship cache
		if (id.equals(P_ID_AUTHORSHIPCACHE)) {
			descriptors.addElement(
					isProtectedCache ?
							new PropertyDescriptor(P_ID_AUTHORSHIPCACHE, P_AUTHORSHIPCACHE) :
							new TextPropertyDescriptor(P_ID_AUTHORSHIPCACHE, P_AUTHORSHIPCACHE));
		}
		
		// Ex-Authorship cache
		if (id.equals(P_ID_EXAUTHORSHIPCACHE)) {
			descriptors.addElement(
					isProtectedCache ?
							new PropertyDescriptor(P_ID_EXAUTHORSHIPCACHE, P_EXAUTHORSHIPCACHE) :						
							new TextPropertyDescriptor(P_ID_EXAUTHORSHIPCACHE, P_EXAUTHORSHIPCACHE));
		}		
		
		// Basionym Authorship cache
		if (id.equals(P_ID_BASAUTHORSHIPCACHE)) {
			descriptors.addElement(
					isProtectedCache ?
							new PropertyDescriptor(P_ID_BASAUTHORSHIPCACHE, P_BASAUTHORSHIPCACHE) :
							new TextPropertyDescriptor(P_ID_BASAUTHORSHIPCACHE, P_BASAUTHORSHIPCACHE));
		}		
		
		// Ex-Basionym Authorship cache
		if (id.equals(P_ID_EXBASAUTHORSHIPCACHE)) {
			descriptors.addElement(
					isProtectedCache ?					
							new PropertyDescriptor(P_ID_EXBASAUTHORSHIPCACHE, P_EXBASAUTHORSHIPCACHE) :
							new TextPropertyDescriptor(P_ID_EXBASAUTHORSHIPCACHE, P_EXBASAUTHORSHIPCACHE));
		}		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return CdmUtils.Nz(name.getAuthorshipCache());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object id) {

		// Editable cache
		if (id.equals(P_ID_EDITABLECACHE)) {
			return CdmUtils.Nz(name.getAuthorshipCache());
		}
		
		// Protect cache?
		if (id.equals(P_ID_PROTECT_CACHE)) {
			return name.isProtectedAuthorshipCache() ? CACHE_PROTECTED : CACHE_NOT_PROTECTED;
		}
		
		// Authorship cache
		if (id.equals(P_ID_AUTHORSHIPCACHE)) {
			if (name.getCombinationAuthorTeam() != null) {
				return CdmUtils.Nz(name.getCombinationAuthorTeam().getNomenclaturalTitle());
			} else {
				return "";
			}
		}
		
		// Ex-Authorship cache
		if (id.equals(P_ID_EXAUTHORSHIPCACHE)) {
			if (name.getExCombinationAuthorTeam() != null) {
				return CdmUtils.Nz(name.getExCombinationAuthorTeam().getNomenclaturalTitle());
			} else {
				return "";
			}
		}		
		
		// Basionym Authorship cache
		if (id.equals(P_ID_BASAUTHORSHIPCACHE)) {
			if (name.getBasionymAuthorTeam() != null) {
				return CdmUtils.Nz(name.getBasionymAuthorTeam().getNomenclaturalTitle());
			} else {
				return "";
			}
		}		
		
		// Ex-Basionym Authorship cache
		if (id.equals(P_ID_EXBASAUTHORSHIPCACHE)) {
			if (name.getExBasionymAuthorTeam() != null) {
				return CdmUtils.Nz(name.getExBasionymAuthorTeam().getNomenclaturalTitle());
			} else {
				return "";
			}
		}	
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(Object id) {}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {

		// Editable cache
		if (id.equals(P_ID_EDITABLECACHE)) {
			name.setAuthorshipCache((String) value);
			name.generateAuthorship();
		}
		
		// Protect cache?
		if (id.equals(P_ID_PROTECT_CACHE)) {
			name.setProtectedAuthorshipCache(
					((Integer) value).intValue() == CACHE_PROTECTED);
		}
		
		// Authorship cache
		if (id.equals(P_ID_AUTHORSHIPCACHE)) {
			if (name.getCombinationAuthorTeam() == null) {
				name.setCombinationAuthorTeam(createNewTeam((String) value));
			} else {
				name.getCombinationAuthorTeam().setNomenclaturalTitle((String) value);
			}
		}
		
		// Ex-Authorship cache
		if (id.equals(P_ID_EXAUTHORSHIPCACHE)) {
			if (name.getExCombinationAuthorTeam() == null) {
				name.setExCombinationAuthorTeam(createNewTeam((String) value));
			} else {
				name.getExCombinationAuthorTeam().setNomenclaturalTitle((String) value);
			}
		}		
		
		// Basionym Authorship cache
		if (id.equals(P_ID_BASAUTHORSHIPCACHE)) {
			if (name.getBasionymAuthorTeam() == null) {
				name.setBasionymAuthorTeam(createNewTeam((String) value));
			} else {
				name.getBasionymAuthorTeam().setNomenclaturalTitle((String) value);
			}
		}		
		
		// Ex-Basionym Authorship cache
		if (id.equals(P_ID_EXBASAUTHORSHIPCACHE)) {
			if (name.getExBasionymAuthorTeam() == null) {
				name.setExBasionymAuthorTeam(createNewTeam((String) value));
			} else {
				name.getExBasionymAuthorTeam().setNomenclaturalTitle((String) value);
			}
		}	
	}

	/**
	 * @param value
	 * @return
	 */
	private INomenclaturalAuthor createNewTeam(String teamCache) {
		Team team = Team.NewInstance();
		team.setNomenclaturalTitle(teamCache);
		return team;
	}
}