/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.name;

import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import eu.etaxonomy.cdm.model.name.ZoologicalName;

/**
 * Property source for ZoologicalName entries.
 *  
 * @author p.ciardelli
 * @created 21.05.2008
 * @version 1.0
 */
public class ZoologicalNamePropertySource extends NonViralNamePropertySource {

	private ZoologicalName zoologicalName;
	
    // Property unique keys
	public static final String P_ID_PUBLICATIONYEAR = "publicationyear";
    // Property display keys
	public static final String P_PUBLICATIONYEAR = "Publication Year";
	
	// Where in the list of properties should the year be placed?
	public static final int P_PUBLICATIONYEAR_POS = 5;
		
	public ZoologicalNamePropertySource(ZoologicalName name) {
		super(name);
		this.zoologicalName = name;
		addDescriptor(P_ID_PUBLICATIONYEAR);
	}

	@Override
	protected void addDescriptor(String id) {
		
		super.addDescriptor(id);
		
		if (id.equals(P_ID_PUBLICATIONYEAR)) {
			PropertyDescriptor descriptor = new TextPropertyDescriptor
					(P_ID_PUBLICATIONYEAR, P_PUBLICATIONYEAR);
			descriptor.setValidator(new IntegerValidator());
			descriptors.add(P_PUBLICATIONYEAR_POS, descriptor);
		}
	}
	
	@Override
	public Object getPropertyValue(Object id) {
        if (id.equals(P_ID_NOMENCLATURAL_CODE)) {
        	return "ICZN";
        }
        if (id.equals(P_ID_PUBLICATIONYEAR)) {
        	if (zoologicalName.getPublicationYear() == null) {
        		return "";
        	}
        	return Integer.toString(zoologicalName.getPublicationYear());
        }
		return super.getPropertyValue(id);
	}
	
	public void setPropertyValue(Object id, Object value) {
		
		if (id.equals(P_ID_PUBLICATIONYEAR)) {
			try {
				zoologicalName.setPublicationYear(yearToInteger((String) value));
			} catch (NumberFormatException e) {
				// Do nothing - validator should prevent this
			}
        }
		
		super.setPropertyValue(id, value);
	}
	
	private Integer yearToInteger(String year) throws NumberFormatException {
		if (year.length() == 0) {
			return null;
		}
		return Integer.parseInt(year);
	}
	
	class IntegerValidator implements ICellEditorValidator {

		public String isValid(Object value) {
			String msg = null;
			try {
				yearToInteger((String) value);
			} catch (NumberFormatException e) {
				msg = "Year contains non-integer characters.";
			}
			return msg;
		}
	}
}