/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.name.TaxonSearchDialog;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.operations.MoveTaxonOperation;

/**
 * @author n.hoffmann
 * @created 01.04.2009
 * @version 1.0
 */
public class MoveTaxonHandler extends AbstractHandler implements IPostOperationEnabled {
	private static final Logger logger = Logger
			.getLogger(MoveTaxonHandler.class);
	private TaxonNode parentTaxonNode;

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		Shell shell = NavigationUtil.getShell();
		TaxonNode taxonNode = (TaxonNode) NavigationUtil.getCurrentSelection(event);
				
		Dialog dialog = new TaxonSearchDialog(shell);
        Object value = ((TaxonSearchDialog) dialog).open();
        
        logger.warn("child " +taxonNode.getTaxon().toString());
        logger.warn("parent " + ((TaxonNode) value).getTaxon().toString());
        
		if (value instanceof TaxonNode) {
			parentTaxonNode = (TaxonNode) value;
			
			if(NavigationUtil.isDirty(parentTaxonNode)){
				MessageDialog.openWarning(shell, "Unsaved Parent Taxon", "There are unsaved " +
						"changes in the parent taxon. Please save first.");
				return null;
			}
			
			IUndoableOperation operation = new MoveTaxonOperation
					("Move taxon to new parent", NavigationUtil.getUndoContext(), 
								taxonNode, parentTaxonNode, this); //$NON-NLS-1$
			NavigationUtil.executeOperation(operation);
		} else {
			logger.warn("TaxonSearchDialog did not return a TaxonNode object.");
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		NavigationUtil.selectInNavigator(objectAffectedByOperation, parentTaxonNode);
		return true;
	}
}
