// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.navigator;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.ui.navigator.CommonDropAdapter;
import org.eclipse.ui.navigator.CommonDropAdapterAssistant;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.model.NameHelper;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.operations.MoveTaxonOperation;

/**
 * @author p.ciardelli
 * @created 03.06.2009
 * @version 1.0
 */
public class TaxonDropAdapterAssistant extends CommonDropAdapterAssistant implements IPostOperationEnabled{
	private static final Logger logger = Logger
			.getLogger(TaxonDropAdapterAssistant.class);

	public static final String ID = "eu.etaxonomy.taxeditor.navigation.navigator.dropassistant"; //$NON-NLS-1$

	private TaxonNode newParentTaxonNode;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonDropAdapterAssistant#handleDrop(org.eclipse.ui.navigator.CommonDropAdapter, org.eclipse.swt.dnd.DropTargetEvent, java.lang.Object)
	 */
	@Override
	public IStatus handleDrop(CommonDropAdapter dropAdapter,
			DropTargetEvent dropTargetEvent, Object target) {
		
		ISelection selection = LocalSelectionTransfer.getTransfer().getSelection();
		if (selection instanceof TreeSelection) {
			Object element = ((TreeSelection) selection).getFirstElement();
			if (element instanceof TaxonNode) {
				TaxonNode childTaxonNode = (TaxonNode) element;
				if (target instanceof TaxonNode) {
					TaxonNode parentTaxonNode = (TaxonNode) target;
					
					return moveTaxon(childTaxonNode, parentTaxonNode);
				}
			}
		}
				
		return Status.CANCEL_STATUS;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonDropAdapterAssistant#validateDrop(java.lang.Object, int, org.eclipse.swt.dnd.TransferData)
	 */
	@Override
	public IStatus validateDrop(Object target, int operation,
			TransferData transferType) {
		if (target instanceof TaxonNode) {
			return Status.OK_STATUS;
		}
		return Status.CANCEL_STATUS;
	}
	

	/**
	 * @param childTaxonNode
	 * @param parentTaxon
	 * @return
	 */
	private IStatus moveTaxon(TaxonNode childTaxonNode, TaxonNode newParentTaxonNode) {
		
		this.newParentTaxonNode = newParentTaxonNode;
		
		// Make sure parentTaxon is not a child
		if (childTaxonNode.getParent().equals(newParentTaxonNode)){
			
			MessageDialog.openError(NavigationUtil.getShell(), "Can't move taxon.",
					"'" + NameHelper.getDisplayName(childTaxonNode.getTaxon()) + "' sits above " +
					"'" + NameHelper.getDisplayName(newParentTaxonNode.getTaxon()) + "' " +
					"in the taxonomic hierarchy.");
			
			return Status.CANCEL_STATUS;
		}
		
		// Make sure taxon is not being dropped onto itself
		if (childTaxonNode.equals(newParentTaxonNode)) {
			return Status.CANCEL_STATUS;
		}
		
		// Make sure parent taxon does not have unsaved changes
		if (NavigationUtil.isDirty(newParentTaxonNode)){
			MessageDialog.openWarning(NavigationUtil.getShell(), "Unsaved Parent Taxon", "There are unsaved " +
			"changes in the parent taxon. Pleas save first.");
			return Status.CANCEL_STATUS;
		}
		
		IUndoContext workspaceUndoContext = NavigationUtil.getWorkbenchUndoContext();
		if (workspaceUndoContext == null) {
			logger.error("Workspace undo context is null. DND operation cancelled");
			return Status.CANCEL_STATUS;
		}

		IUndoableOperation operation = new MoveTaxonOperation
				("Move Taxon", workspaceUndoContext, childTaxonNode, newParentTaxonNode, this);
		NavigationUtil.executeOperation(operation);	
		
		
		logger.info("Moved taxon " + childTaxonNode + " to new parent " + newParentTaxonNode);
		
		NavigationUtil.selectInNavigator(childTaxonNode, newParentTaxonNode);
		
		return Status.OK_STATUS;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		NavigationUtil.selectInNavigator(objectAffectedByOperation, newParentTaxonNode);
		return true;
	}
}
