/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.navigation.taxonomictree;

import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.part.ViewPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.DataChangeBridge;
import eu.etaxonomy.taxeditor.model.IDataChangeBehavior;
import eu.etaxonomy.taxeditor.model.TaxonTransfer;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * The panel containing the taxonomic tree.
 * 
 * @author p.ciardelli
 * @created 27.05.2008
 * @version 1.0
 * @author n.hoffmann
 */
@Deprecated 
public class TaxonomicTreeView extends ViewPart implements IConversationEnabled, IPostOperationEnabled {
	private static final Logger logger = Logger.getLogger(TaxonomicTreeView.class);
	
	public static final String ID = "eu.etaxonomy.taxeditor.navigation.taxonomictree.treeView"; //$NON-NLS-1$
	
	private TreeViewer treeViewer;

	private ConversationHolder conversation;

	private IDataChangeBehavior dataChangeBehavior;

	private Composite parent;
	
	/**
	 * Create contents of the view part
	 * @param parent
	 */
	@Override
	public void createPartControl(Composite parent) {
		conversation = CdmStore.NewTransactionalConversation();
		conversation.registerForDataStoreChanges(this);

		
		if(this.parent == null){
			this.parent = parent;
		}
		
		// Create top composite
		FillLayout fillLayout = new FillLayout();
		fillLayout.marginWidth = 0;
		fillLayout.marginHeight = 0;
		fillLayout.type = SWT.VERTICAL;	
		parent.setLayout(fillLayout);
		
		createTreeViewer(CdmStore.getDefault().getRootTaxa());
	}
		
	/**
	 * Actual tooltips built in label provider
	 */
	private void createToolTips() {
		ColumnViewerToolTipSupport.enableFor(treeViewer);
	}
	
	
	public void createTreeViewer(List<Taxon> rootTaxa) {	
		
		// Create tree
		treeViewer = new TreeViewer(parent, SWT.VIRTUAL);
		
		Tree tree = treeViewer.getTree();
		
		tree.setLinesVisible(false);
		tree.setHeaderVisible(false);
		
		treeViewer.setContentProvider(new TaxonomicTreeContentProvider());
		treeViewer.setLabelProvider(new TaxonomicTreeLabelProvider());
		
		treeViewer.setInput(rootTaxa);
		
		createDoubleClickListener();
		createToolTips();
		createDragAndDrop();
		
		// Sort according to "getColumnText" above, i.e. by NameCache
		// TODO check if this is still up to date
		treeViewer.setComparator(new ViewerComparator());
	
		// register context menu
		MenuManager menuMgr = new MenuManager();
		menuMgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		getSite().registerContextMenu(menuMgr, treeViewer);

		Control control = treeViewer.getControl();
		Menu menu = menuMgr.createContextMenu(control);
		control.setMenu(menu);	
			
	}
	
	/**
	 * On double click, open name editor
	 */
	public void createDoubleClickListener() {
		treeViewer.addDoubleClickListener(new IDoubleClickListener() {

			public void doubleClick(DoubleClickEvent event) {
				if (event.getSelection() instanceof StructuredSelection) {
					Object element = ((StructuredSelection) event
							.getSelection()).getFirstElement();
					if (element instanceof Taxon) {
						// FIXME call EditTaxonHandler
						logger.info("Doubleclick on taxon element. Should open editor now");
//						NavigationUtil.openEditor((Taxon) element);
					}
				}
			}
		});
	}
	
	/**
	 * 
	 */
	public void createDragAndDrop() {
		final Tree tree = treeViewer.getTree();
		Transfer[] types = new Transfer[] { TaxonTransfer.getInstance() };
		int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK;

		final DragSource source = new DragSource(tree, operations);
		source.setTransfer(types);
		final TreeItem[] dragSourceItem = new TreeItem[1];
		source.addDragListener(new DragSourceAdapter() {
			public void dragStart(DragSourceEvent event) {
				TreeItem[] selection = tree.getSelection();
				if (selection.length > 0) {
					event.doit = true;
					dragSourceItem[0] = selection[0];
				} else {
					event.doit = false;
				}
			}

			public void dragSetData(DragSourceEvent event) {
				Taxon taxon = (Taxon) dragSourceItem[0].getData();
				TaxonTransfer.getInstance().setTaxon(taxon);
			}
		});

		DropTarget target = new DropTarget(tree, operations);
		target.setTransfer(types);
		target.addDropListener(new DropTargetAdapter() {
			public void drop(DropTargetEvent event) {

				// Only drop ONTO another TreeItem OR outside the tree area.
				// Dropping between TreeItems makes no sense because tree is
				// sorted.
				Taxon taxon = (Taxon) dragSourceItem[0].getData();
				Taxon newParentTaxon = null;

				// If event.item == null, user has dragged outside of the tree
				if (event.item == null) {
					newParentTaxon = null;
				} else {
					newParentTaxon = (Taxon) event.item.getData();
					
					// Make sure parentTaxon is not a child
//					if (TaxonNodeUtil.isTaxonChildOfTaxon(newParentTaxon, taxon)) {
//						
//						MessageDialog.openError(NavigationUtil.getShell(), "Can't move taxon.",
//								"'" + NameUtil.getDisplayName(taxon) + "' sits above " +
//								"'" + NameUtil.getDisplayName(newParentTaxon) + "' " +
//								"in the taxonomic hierarchy.");
//						
//						return;
//					}
					
					// Make sure taxon is not being dropped onto itself
					if (taxon.equals(newParentTaxon)) {
						return;
					}
					
					// Make sure parent taxon does not have unsaved changes
//					if (NavigationUtil.isDirty(newParentTaxon)){
//						MessageDialog.openWarning(NavigationUtil.getShell(), "Unsaved Parent Taxon", "There are unsaved " +
//						"changes in the parent taxon. Pleas save first.");
//						return;
//					}
				}

//				
//				IUndoableOperation operation = new MoveTaxonOperation
//						("Move Taxon", NavigationUtil.getUndoContext(), taxon, newParentTaxon);
//				
//				NavigationUtil.executeOperation(operation);			
			}
		});
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	public void setFocus() {
		// bind resources for this widget on focus
		getConversationHolder().bind();
		// pass focus to the tree viewer if it exists
		if(treeViewer != null){
			treeViewer.getControl().setFocus();
		}
	}
	
	/**
	 * Expose TreeViewer, i.e. to manipulate tree nodes
	 * @return
	 */
	public TreeViewer getTreeViewer() {
		return treeViewer;
	}
	
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostCrudObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmCrudEvent)
	 */
	public void update(CdmDataChangeMap changeEvents) {
		if(dataChangeBehavior == null){
			dataChangeBehavior = new TaxonomicTreeViewDataChangeBehaviour(this);
		}
		DataChangeBridge.handleDataChange(changeEvents, dataChangeBehavior);
	}

	/**
	 * Start the creation process of this Control anew. 
	 */
	public void recreate() {
		this.createPartControl(parent);
	}
	
	/**
	 * Get a fresh model from CdmStore and refresh the tree viewer with the data
	 */
	public void refresh(){
		treeViewer.setInput(CdmStore.getDefault().getRootTaxa());
		treeViewer.refresh();		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#dispose()
	 */
	@Override
	public void dispose() {
		conversation.unregisterForDataStoreChanges(this);

		super.dispose();
	}

	/**
	 * If taxon node in the tree is hidden, open and select it
	 * 
	 * @param taxon
	 */
	public void revealTaxon(Taxon taxon) {
		if (taxon == null) {
			return;
		}
		treeViewer.setSelection(new StructuredSelection(taxon), true);
		treeViewer.reveal(taxon);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.store.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		
		if(objectAffectedByOperation instanceof Taxon){
			revealTaxon((Taxon) objectAffectedByOperation);
		}
		
		return true;
	}
	
	
}
