// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.dialogs;

import java.text.Collator;
import java.util.Comparator;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.dialogs.FilteredItemsSelectionDialog;

import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.strategy.cache.common.IIdentifiableEntityCacheStrategy;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author n.hoffmann
 * @created 04.06.2009
 * @version 1.0
 */
public abstract class FilteredCdmResourceSelectionDialog extends
		FilteredItemsSelectionDialog {
	
	/**
	 * @param shell
	 */
	public FilteredCdmResourceSelectionDialog(Shell shell, String title) {
		super(shell);
		this.setTitle(title);
		this.setListLabelProvider(new SelectionDialogSelectionLabelProvider());
		this.setDetailsLabelProvider(new SelectionDialogDetailsLabelProvider());
		this.setSelectionHistory(new CdmResourceHistory());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#getDialogSettings()
	 */
	@Override
	protected IDialogSettings getDialogSettings() {
		IDialogSettings settings = TaxeditorStorePlugin.getDefault().getDialogSettings().getSection(getSettings());

		if (settings == null) {
			settings = TaxeditorStorePlugin.getDefault().getDialogSettings().addNewSection(getSettings());
		}
		return settings;
	}
	
	/**
	 * 
	 * @return
	 */
	public abstract String getSettings();
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#getElementName(java.lang.Object)
	 */
	@Override
	public String getElementName(Object item) {
		return ((IdentifiableEntity<IIdentifiableEntityCacheStrategy>) item).getTitleCache();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#validateItem(java.lang.Object)
	 */
	@Override
	protected IStatus validateItem(Object item) {
		return Status.OK_STATUS;
	}
	

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#getItemsComparator()
	 */
	@Override
	protected Comparator getItemsComparator() {
		return new Comparator<IdentifiableEntity>() {
			public int compare(IdentifiableEntity entity1, IdentifiableEntity entity2) {
				Collator collator = Collator.getInstance();
				return collator.compare(entity1.getTitleCache(), entity2.getTitleCache());
			}
		};
	}		
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createFilter()
	 */
	@Override
	protected ItemsFilter createFilter() {
		return new CdmResourceFilter();
	}
	
	/**
	 * 
	 * @author n.hoffmann
	 * @created 04.06.2009
	 * @version 1.0
	 */
	class CdmResourceFilter extends ItemsFilter {

		@Override
		public boolean isConsistentItem(Object item) {
			return true; // don't remember the previous selection.
		}

		@Override
		public boolean matchItem(Object item) {
			return matches(((IdentifiableEntity<IIdentifiableEntityCacheStrategy>) item).getTitleCache());
		}
	}
	
	/**
	 * 
	 * @author n.hoffmann
	 * @created 04.06.2009
	 * @version 1.0
	 */
	class CdmResourceHistory extends SelectionHistory{

		@Override
		protected Object restoreItemFromMemento(IMemento memento) {
			return null;
		}

		@Override
		protected void storeItemToMemento(Object item, IMemento memento) {

		}

	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createExtendedContentArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		return null;
	}
}
