// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.taxon.TaxonomicTree;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author n.hoffmann
 * @created 08.05.2009
 * @version 1.0
 */
public class CreateChildTaxonOperation extends AbstractPostOperation {
	
	private static final Logger logger = Logger.getLogger(CreateChildTaxonOperation.class);
	
	private TaxonNameBase<?, ?> newTaxonName;

	private TaxonNode childTaxonNode;

	
	/**
	 * @param label
	 * @param undoContext
	 * @param taxon
	 * @param postOperationEnabled
	 * @param conversationEnabled
	 */
	public CreateChildTaxonOperation(String label, IUndoContext undoContext,
			UUID parentNodeUuid, TaxonNameBase<?, ?> name, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, parentNodeUuid, postOperationEnabled);
		
		this.newTaxonName = name;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		childTaxonNode = addChildTaxonBaseIsolated(parentNodeUuid, newTaxonName);
		
		if(childTaxonNode != null){
			return postExecute(childTaxonNode);
		}else{
			return new Status(IStatus.ERROR, TaxeditorStorePlugin.PLUGIN_ID, TaxeditorStorePlugin.ERROR_SAVING_ZERO_NAME, "Attempt to save taxon with zero length name", null);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		logger.warn("Not yet implemented.");
		return null;
	}
	
	private TaxonNode addChildTaxonBaseIsolated(UUID parentNodeUuid, TaxonNameBase<?, ?> newTaxonName){
		// disallow saving of zero length names
		if(newTaxonName.getFullTitleCache().length() == 0){
			return null;
		}
		
		// get a new conversation
		ConversationHolder conversation = CdmStore.NewTransactionalConversation();
		// find the taxonBase
		TaxonNode parentTaxonNode = CdmStore.getTaxonService().getTaxonNodeByUuid(parentNodeUuid);
		
		Taxon childTaxon;
		TaxonNode childTaxonNode;
		
		if(parentTaxonNode == null){
			// parent seems to of type TaxonomicTree
			TaxonomicTree tree = CdmStore.getTaxonService().getTaxonomicTreeByUuid(parentNodeUuid);
			childTaxon = Taxon.NewInstance(newTaxonName, CdmStore.getDefault().getDefaultSec());
			
			childTaxonNode = tree.addRoot(childTaxon, null);
		}else{
			// logic to add new taxon name
			childTaxon = Taxon.NewInstance(newTaxonName, parentTaxonNode.getTaxon().getSec());
			childTaxonNode = parentTaxonNode.addChild(childTaxon);
		}
		// commit the conversation and throw it away
		conversation.commit(true);
		return childTaxonNode;
	}
}
