// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference.wizards;

import org.apache.log4j.Logger;
import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.VocabularyStore;

/**
 * @author n.hoffmann
 * @created 12.06.2009
 * @version 1.0
 */
public class FeatureWizard extends Wizard {
	private static final Logger logger = Logger.getLogger(FeatureWizard.class);
	
	private Feature feature = null;
	
	private boolean editMode = false;

	/**
	 * 
	 */
	public FeatureWizard(){}
	
	/**
	 * 
	 * @param selectedFeature
	 */
	public FeatureWizard(Feature selectedFeature){
		this.feature = selectedFeature;
		editMode = true;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {
		super.addPages();
		
		addPage(new FeatureWizardPage());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {

		ConversationHolder conversation = CdmStore.NewTransactionalConversation();
		
		if(editMode){
			VocabularyStore.updateVocabulary(VocabularyEnum.Feature);
		}else{
			VocabularyStore.saveTerm(feature, VocabularyEnum.Feature);
		}
		
		conversation.commit(true);
		
		return true;
	}


	/**
	 * @param text
	 * @param text2
	 * @param text3
	 */
	public void setFeature(String term, String label, String labelAbbreviation) {
		feature = Feature.NewInstance(term, label, labelAbbreviation);
	}

	/**
	 * @return the feature
	 */
	public Feature getFeature() {
		return feature;
	}

	/**
	 * @return the editMode
	 */
	public boolean isEditMode() {
		return editMode;
	}
}
