// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference.wizards;

import org.apache.log4j.Logger;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.description.Feature;

/**
 * @author n.hoffmann
 * @created 12.06.2009
 * @version 1.0
 */
public class FeatureWizardPage extends WizardPage implements ModifyListener {

	private Text text_label;
	private Text text_labelAbbreviation;
	private Text text_description;

	
	
	/**
	 * 
	 */
	public FeatureWizardPage() {
		super("FEATURE_WIZARD_PAGE");		
	}

	private static final Logger logger = Logger
			.getLogger(FeatureWizardPage.class);

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		String title = getWizard().isEditMode() ? "Edit Feature" : "Create new Feature";
		this.setTitle(title);
		
		Composite composite = new Composite(parent, SWT.NULL);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		gridLayout.horizontalSpacing = SWT.FILL;
		gridLayout.verticalSpacing = SWT.FILL;
		composite.setLayout(gridLayout);
		
		// label
		CLabel label_label = new CLabel(composite, SWT.NULL);
		label_label.setText("Label");
		
		text_label = new Text(composite, SWT.BORDER);
		text_label.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		text_label.addModifyListener(this);
		
		// label abbreviation 
		CLabel label_labelAbbreviation = new CLabel(composite, SWT.NULL);
		label_labelAbbreviation.setText("Label Abbreviation");
		
		text_labelAbbreviation = new Text(composite, SWT.BORDER);
		text_labelAbbreviation.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		text_labelAbbreviation.addModifyListener(this);
		
		// descriptive text
		CLabel label_description = new CLabel(composite, SWT.NULL);
		label_description.setText("Description");
		label_description.setLayoutData(new GridData(SWT.TOP));
		
		text_description = new Text(composite, SWT.BORDER | SWT.MULTI);
		text_description.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		text_description.addModifyListener(this);
		
		setControl(composite);
		
		if(getWizard().getFeature() != null){
			Feature feature = getWizard().getFeature();
			text_label.setText(feature.getLabel());
			// why can't we access labelAbbreviation
//			text_labelAbbreviation.setText(feature.get)
			// why can't we access the term string?
//			text_description.setText(feature.getT)
		}
	}


	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		if(text_description.getText().length() > 0 &&
				text_labelAbbreviation.getText().length() > 0 &&
				text_label.getText().length() > 0 ){
			setPageComplete(true);
			((FeatureWizard) getWizard()).setFeature(text_description.getText(), text_label.getText(), text_labelAbbreviation.getText());
		}
	}


	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.WizardPage#getWizard()
	 */
	@Override
	public FeatureWizard getWizard() {
		return (FeatureWizard) super.getWizard();
	}
	
	
}
